from .paggf_utils import PAGGFUtils, get_background_layer
from . import paggf_version
import os.path
from .layout.doc_gen import LayoutValidator
from PyQt5.QtCore import QLocale
from .db_store_layout_items import Loader as StoreLayoutLoader


class SketchGenerator():
    # config = config = self.actions_configuration[action_name]
    def __init__(self, version, config, iface, layers):
        self.version = version
        self.config = config
        self.iface = iface
        self.layers = layers
        self.utils = PAGGFUtils(self.layers)

    def get_generate_and_validate_parcel_sketch(self):
        def slot(layer, point_clicked, feature):
            if self.version == paggf_version.BM:
                output_filename = '{}_{}_CROQUIS_BM'.format(
                    feature['idc_parcel_code'], feature['hill_district_name'])
                template_file = 'layout/sketch_bm_A4_vlayout.qpt'
            elif self.version == paggf_version.CID:
                output_filename = '{}_{}_CROQUIS_CID'.format(
                    feature['idc_parcel_code'], feature['hill_district_name'])
                template_file = 'layout/sketch_cid_A4_vlayout.qpt'
            else:
                output_filename = '{}_{}_CROQUIS_EFTF'.format(
                    feature['idc_parcel_code'], feature['hill_district_name'])
                template_file = 'layout/sketch_other_A4_vlayout.qpt'

            template_path = os.path.join(
                self.utils.plugin_dir, template_file)

            self.cv = LayoutValidator(
                feature, template_path, self.iface, zoom_to_feature=True,
                scale_step=250, output_png=True, output_pdf=True,
                output_filename=output_filename, load_feature_in_sub_map=True)

            self.cv.version = self.version

            if 'export_after_generation' in self.config and self.config['export_after_generation']:
                # affichage du PV en pdf
                pass
            else:
                self.cv.callback = self.cv.get_callback_to_store_into_feature(
                    layer, feature, 'doc_pdf_parcel_sketch',
                    'doc_png_parcel_sketch')

            self.cv.add_substitutions({
                'NORTH_ARROW_PATH': os.path.join(
                    self.utils.plugin_dir, 'img/north.png'),
            })

            if 'commission_reference' in feature:
                self.cv.add_substitution(
                    'cid_number', feature['commission_reference'])

            if self.version == paggf_version.CID:
                layers_config_carto = []

                for layer, style, style_substitution_map in [
                    (
                        self.layers.get('gis', 'idc_parcels_to_valid'),
                        os.path.join(
                            self.utils.plugin_dir, "layer_style/idc_only_current_parcel_sketch_A4.qml"),
                        {'current_parcel_id': lambda f: f['id']}),
                    (
                        self.layers.get('gis', 'current_parcels'),
                        os.path.join(
                            self.utils.plugin_dir, "layer_style/idc_only_current_parcel_sketch_A4.qml"),
                        {'current_parcel_id': lambda f: f['id']}),
                    (
                        self.layers.get('gis', 'idc_vertices_to_valid'),
                        os.path.join(
                            self.utils.plugin_dir, "layer_style/idc_only_current_vertex_sketch_A4.qml"),
                        {'current_parcel_id': lambda f: f['id']}),
                    (
                        self.layers.get('gis', 'current_idc_vertices'),
                        os.path.join(
                            self.utils.plugin_dir, "layer_style/idc_only_current_vertex_sketch_A4.qml"),
                        {'current_parcel_id': lambda f: f['id']}),
                ]:
                    if layer is not None:
                        layers_config_carto.append((layer, style, style_substitution_map))

                if 'with_ortho' in self.config and self.config['with_ortho']:
                    layers_config_carto.append((get_background_layer(), None))

                self.cv.layers_config = {
                    'carto': layers_config_carto
                }

            elif self.version == paggf_version.BM:
                fr_locale = QLocale(QLocale.French, QLocale.France)
                if feature['end_date']:
                    delivery_date_day_month = fr_locale.toString(
                        feature['end_date'], 'MMMM yyyy')
                else:
                    delivery_date_day_month = 'NULL'

                self.cv.add_substitution(
                    'place_and_date',
                    "{}, {}".format(
                        feature['municipality_name'], delivery_date_day_month))
                self.cv.add_substitution(
                    'calculated_area_m2_for_sketch',
                    "{:.2f}".format(feature['calculated_area_m2']))

                self.cv.layers_config = {
                    'carto': [
                        (
                            self.layers.get('gis', 'parcels_to_valid'),
                            os.path.join(
                                self.utils.plugin_dir,
                                "layer_style/bm_only_current_parcel_A4.qml"),
                            {'current_parcel_idc_code': lambda f: f['idc_parcel_code']}),
                        (
                            self.layers.get('gis', 'vertices_to_valid'),
                            os.path.join(
                                self.utils.plugin_dir,
                                "layer_style/bm_only_current_vertex_A4.qml"),
                            {'current_parcel_id': lambda f: f['id']}),
                        (self.layers.get(
                            'gis', 'parcel_buildings_to_valid'), os.path.join(
                                self.utils.plugin_dir,
                                "layer_style/bm_building_A4.qml")),
                        (self.layers.get(
                            'gis', 'parcel_rivers_to_valid'), os.path.join(
                                self.utils.plugin_dir,
                                "layer_style/bm_river_A4.qml")),
                        (self.layers.get(
                            'gis', 'parcel_roads_to_valid'), os.path.join(
                                self.utils.plugin_dir,
                                "layer_style/bm_road_A4.qml")),
                        (
                            self.layers.get('gis', 'current_parcels'),
                            os.path.join(
                                self.utils.plugin_dir,
                                "layer_style/bm_only_current_parcel_A4.qml"),
                            {'current_parcel_idc_code': lambda f: f['idc_parcel_code']}),
                        (
                            self.layers.get('gis', 'current_vertices'),
                            os.path.join(
                                self.utils.plugin_dir,
                                "layer_style/bm_only_current_vertex_A4.qml"),
                            {'current_parcel_id': lambda f: f['id']}),
                        (self.layers.get(
                            'gis', 'current_parcel_buildings'), os.path.join(
                                self.utils.plugin_dir,
                                "layer_style/bm_building_A4.qml")),
                        (self.layers.get(
                            'gis', 'current_parcel_rivers'), os.path.join(
                                self.utils.plugin_dir,
                                "layer_style/bm_river_A4.qml")),
                        (self.layers.get(
                            'gis', 'current_parcel_roads'), os.path.join(
                                self.utils.plugin_dir,
                                "layer_style/bm_road_A4.qml")),
                    ]
                }

            if 'scale_step' in self.config:
                self.cv.scale_step = self.config['scale_step']

            if 'zoom_to_feature_scale' in self.config:
                self.cv.zoom_to_feature_scale = self.config['zoom_to_feature_scale']

            self.cv.exec_()

            # for "Préparer et exporter/imprimer un extrait de\croquis de parcelle ID"
            if 'do_not_load_from_db' in self.config \
                    and self.config['do_not_load_from_db']:
                pass
            else:
                db_layout_storage = StoreLayoutLoader.get(self.version)
                db_layout_storage.load(feature, self.cv.layout)

        return slot
