from PyQt5.QtCore import QDate
from PyQt5.QtWidgets import QLabel, QLineEdit, QComboBox, \
    QListWidget, QPushButton, QDialogButtonBox, QDateEdit
from PAGGF.paggf_layers import LoadedLayers
from PAGGF.db import quote_nullable
from qgis.core import QgsExpression, QgsFeatureRequest
from PAGGF.ui.form_helper import fill_combobox_with_layer_content, \
    QListWidgetEditor, get_widget_dict

layers = LoadedLayers()

location_handler = None

class LocationHandler:
    """
    Class to handle location combo boxes
    """
    def __init__(self, dialog, layer, feature):
        self.province_field = dialog.findChild(QComboBox, "province_field")
        self.municipality_field = dialog.findChild(QComboBox, "municipality_field")
        self.hill_field = dialog.findChild(QComboBox, "hill_field")
        self.subhill_field = dialog.findChild(QComboBox, "subhill_field")
        self.layers = layers

        self.provinces_idx = {}
        self.municipalities_idx = {}
        self.hills_idx = {}
        self.sub_hills_idx = {}
        
        # populate combos
        province_layer = self.layers.get("ref", "provinces")
        if province_layer:
            idx = 0
            req = QgsFeatureRequest().setSubsetOfAttributes([province_layer.fields().indexFromName(x) for x in ["id", "name"]]).addOrderBy("id")
            for f in province_layer.getFeatures(req):
                self.province_field.addItem(
                    "{} - {}".format(f['id'], f['name']), f['id'])
                self.provinces_idx[f['id']] = idx
                idx += 1
        self.province_field.activated[int].connect(
            self.when_province_selection_changed)
        self.municipality_field.activated[int].connect(
            self.when_municipality_selection_changed)
        self.hill_field.activated[int].connect(
            self.when_hill_selection_changed)

        # select the current sub hill
        id = feature['id_sub_hill_avenue']

        self.province_field.setCurrentIndex(self.provinces_idx[id[0:2]])
        self.when_province_selection_changed(self.provinces_idx[id[0:2]])
        self.municipality_field.setCurrentIndex(self.municipalities_idx[id[0:4]])
        self.when_municipality_selection_changed(self.municipalities_idx[id[0:4]])
        self.hill_field.setCurrentIndex(self.hills_idx[id[0:6]])
        self.when_hill_selection_changed(self.hills_idx[id[0:6]])

    def when_province_selection_changed(self, index):
        self.municipalities_idx = {}
        mun_layer = self.layers.get("ref", "municipalities")
        self.municipality_field.clear()
        if mun_layer:
            idx = 0
            req = QgsFeatureRequest().setSubsetOfAttributes([mun_layer.fields().indexFromName(x) for x in ["id", "name"]])
            for f in mun_layer.getFeatures(req):
                if f['id'].startswith(self.province_field.currentData()):
                    self.municipality_field.addItem(
                        "{} - {}".format(f['id'], f['name']), f['id'])
                    self.municipalities_idx[f['id']] = idx
                    idx += 1
        self.when_municipality_selection_changed(0)

    def when_municipality_selection_changed(self, index):
        self.hills_idx = {}
        hill_layer = self.layers.get("ref", "hill_districts")
        self.hill_field.clear()
        if hill_layer:
            idx = 0
            req = QgsFeatureRequest().setSubsetOfAttributes([hill_layer.fields().indexFromName(x) for x in ["id", "name"]])
            for f in hill_layer.getFeatures(req):
                if f['id'].startswith(self.municipality_field.currentData()):
                    self.hill_field.addItem(
                        "{} - {}".format(f['id'], f['name']), f['id'])
                    self.hills_idx[f['id']] = idx
                    idx += 1
        self.when_hill_selection_changed(0)

    def when_hill_selection_changed(self, index):
        self.sub_hills_idx = {}
        hill_id = self.hill_field.itemData(index)
        self.subhill_field.clear()
        sub_hill_layer = self.layers.get("ref", "sub_hill_avenues")
        if sub_hill_layer:
            idx = 0
            req = QgsFeatureRequest().setSubsetOfAttributes([sub_hill_layer.fields().indexFromName(x) for x in ["id", "name"]])
            for f in sub_hill_layer.getFeatures(req):
                if f['id'].startswith(self.hill_field.currentData()):
                    self.subhill_field.addItem(
                        "{} - {}".format(f['id'], f['name']), f['id'])
                    self.sub_hills_idx[f['id']] = idx
                    idx += 1

    def subhill(self):
        return self.subhill_field.currentData()
        

def on_form_load(dialog, layer, feature):
    """
    Function executed when inventory form is loaded
    """
    if feature.attributes() == []:
        # form loaded without data, bypass
        return

    origin_other = dialog.findChild(QLineEdit, 'origin_other')
    origin_other_label = dialog.findChild(QLabel, 'origin_other_label')
    ownership_origin = dialog.findChild(QComboBox, 'ownership_origin')

    manager_other = dialog.findChild(QLineEdit, 'manager_other')
    manager_other_label = dialog.findChild(QLabel, 'manager_other_label')
    manager = dialog.findChild(QComboBox, 'manager')

    usage_other = dialog.findChild(QLineEdit, 'usage_other')
    usage_other_label = dialog.findChild(QLabel, 'usage_other_label')
    usage = dialog.findChild(QComboBox, 'usage')

    assignment_other = dialog.findChild(QLineEdit, 'assignment_other')
    assignment_other_label = dialog.findChild(QLabel, 'assignment_other_label')
    assignment = dialog.findChild(QComboBox, 'assignment')

    owner_other = dialog.findChild(QLineEdit, 'owner_other')
    owner_other_label = dialog.findChild(QLabel, 'owner_other_label')
    owner = dialog.findChild(QComboBox, 'owner')

    manager_other = dialog.findChild(QLineEdit, 'manager_other')
    manager_other_label = dialog.findChild(QLabel, 'manager_other_label')
    manager = dialog.findChild(QComboBox, 'manager')

    dialog.findChild(QDateEdit, "begin_date").setMaximumDate(QDate.currentDate())
    dialog.findChild(QDateEdit, "end_date").setMaximumDate(QDate.currentDate())

    global location_handler
    location_handler = LocationHandler(dialog, layer, feature)

    dialog.findChild(QLineEdit, "id_sub_hill_avenue").hide()

    def create_slot_for_field(
            field, value_other, field_other, field_other_label, check_equals=True):
        def value_changed_slot():
            if (check_equals and field.currentData() == value_other) \
                    or ((not check_equals) and field.currentData() != value_other):
                field_other.show()
                field_other_label.show()
            else:
                field_other.hide()
                field_other_label.hide()
        return value_changed_slot

    def on_accept():
        id_sub_hill_field = dialog.findChild(QLineEdit, "id_sub_hill_avenue")
        id_sub_hill_field.setText(location_handler.subhill())

    ok_cancel = dialog.findChild(QDialogButtonBox, "buttonBox")
    ok_cancel.accepted.connect(on_accept)

    for field, value_other, field_other, field_other_label, check_equals in [
        (owner, 'State', owner_other, owner_other_label, False),
        (manager, 'State', manager_other, manager_other_label, False),
        (ownership_origin, 'Other', origin_other, origin_other_label, True),
        (usage, 'Other', usage_other, usage_other_label, True),
        (assignment, 'Other', assignment_other, assignment_other_label, True)
    ]:
        field_other.hide()
        field_other_label.hide()

        value_changed_slot = create_slot_for_field(
            field, value_other, field_other, field_other_label,
            check_equals=check_equals)
        field.currentTextChanged.connect(value_changed_slot)

    adjacent_people_tab_init(dialog, feature)
    commission_member_tab_init(dialog, feature)
    crc_commission_member_tab_init(dialog, feature)
    witness_tab_init(dialog, feature)


def adjacent_people_tab_init(dialog, inventory):
    """
    Initialize the user interaction for the tad 'Adjacent-people'
    """
    global adjacent_people_list_editor

    inventory_id = inventory['id_inventory']

    widget = get_widget_dict(dialog, [
        ('adjacent_people_list', QListWidget),
        ('adjacent_people_firstname', QLineEdit),
        ('adjacent_people_lastname', QLineEdit),
        ('adjacent_people_add', QPushButton),
        ('adjacent_people_delete', QPushButton)])

    adjacent_people_layer = layers.get('idc', 'inventories_adjacent_people')
    #filter_exp = QgsExpression('"id_inventory" = \'{}\''.format(inventory_id))
    #filter_req = QgsFeatureRequest(filter_exp)

    def get_new_data():
        ret = (
            widget['adjacent_people_lastname'].text(),
            widget['adjacent_people_firstname'].text())
        widget['adjacent_people_lastname'].setText('')
        widget['adjacent_people_firstname'].setText('')
        return ret

    def get_data_repr(data):
        return "{} {}".format(data[0], data[1])

    def get_init_data_list():
        ret = []
        filter_exp = QgsExpression(
            '"id_inventory" = \'{}\''.format(inventory_id))
        filter_req = QgsFeatureRequest(filter_exp)
        layer = layers.get('idc', 'inventories_adjacent_people')

        for feature in layer.getFeatures(filter_req):
            ret.append((feature['lastname'], feature['firstname']))
        return ret

    def save_data_fct(data_list):
        if len(data_list) == 0:
            row_people = ""
        else:
            row_people = "row(" + \
                         "),row(".join(["{}, {}".format(quote_nullable(d[1]),
                                                        quote_nullable(d[0]))
                                        for d in data_list]) + ")"
        sql="SELECT gis.inventory_set_adjacent_people('{}', array[{}]::gis._adjacent_people_param[])".format(
            inventory_id, row_people)
        adjacent_people_layer.dataProvider().transaction().executeSql(sql)

    adjacent_people_list_editor = QListWidgetEditor(
        widget['adjacent_people_list'],
        widget['adjacent_people_add'],
        widget['adjacent_people_delete'],
        get_new_data,
        get_data_repr,
        get_init_data_list,
        save_data_fct)


def member_with_quality_tab_init(
        dialog, inventory,
        list_widget_name, firstname_field_name, lastname_field_name,
        add_button_name, quality_field_name,
        delete_button_name, db_table_name,
        quality_field_is_lineedit=True,  # otherwise it is a combobox
        quality_db_table_name=None,
        db_function_name=None):
    inventory_id = inventory['id_inventory']

    if quality_field_is_lineedit:
        quality_field_type = QLineEdit
    else:
        quality_field_type = QComboBox

    widget = get_widget_dict(dialog, [
        (list_widget_name, QListWidget),
        (firstname_field_name, QLineEdit),
        (lastname_field_name, QLineEdit),
        (add_button_name, QPushButton),
        (delete_button_name, QPushButton),
        (quality_field_name, quality_field_type)])

    if quality_db_table_name:
        fill_combobox_with_layer_content(
            widget[quality_field_name],
            layers.get('idc', quality_db_table_name),
            'name',
            'id'
        )

    def get_new_data():
        if quality_field_is_lineedit:
            ret = (
                widget[lastname_field_name].text(),
                widget[firstname_field_name].text(),
                widget[quality_field_name].text(),
                widget[quality_field_name].text())
            widget[quality_field_name].setText('')
        else:
            ret = (
                widget[lastname_field_name].text(),
                widget[firstname_field_name].text(),
                widget[quality_field_name].currentText(),
                widget[quality_field_name].currentData())
        widget[lastname_field_name].setText('')
        widget[firstname_field_name].setText('')
        return ret

    def get_data_repr(data):
        return "{} {} ({})".format(data[0], data[1], data[2])

    def get_init_data_list():
        ret = []
        filter_exp = QgsExpression(
            '"id_inventory" = \'{}\''.format(inventory_id))
        filter_req = QgsFeatureRequest(filter_exp)
        layer = layers.get('idc', db_table_name)

        for feature in layer.getFeatures(filter_req):
            if quality_db_table_name:
                ret.append((
                    feature['lastname'],
                    feature['firstname'],
                    feature['member_quality_id'],
                    feature['member_quality_id']))
            else:
                ret.append((
                    feature['lastname'],
                    feature['firstname'],
                    feature['quality'], feature['quality']))
        return ret

    def save_data_fct(data_list):
        layer = layers.get('idc', db_table_name)

        if len(data_list) == 0:
            row_people = ""
        else:
            row_people = "row(" + \
                         "),row(".join(["{}, {}, {}".format(quote_nullable(d[0]),
                                                            quote_nullable(d[1]),
                                                            quote_nullable(d[3]))
                                        for d in data_list]) + ")"
        sql="SELECT gis.{}('{}', array[{}]::gis._{}_param[])".format(
            db_function_name, inventory_id, row_people, db_function_name)
        layer.dataProvider().transaction().executeSql(sql)

    return QListWidgetEditor(
        widget[list_widget_name],
        widget[add_button_name],
        widget[delete_button_name],
        get_new_data,
        get_data_repr,
        get_init_data_list,
        save_data_fct)


def commission_member_tab_init(dialog, inventory):
    global commission_members_list
    commission_members_list = member_with_quality_tab_init(
        dialog, inventory,
        'commission_members_list', 'commission_members_firstname',
        'commission_members_lastname', 'commission_members_add',
        'commission_members_quality', 'commission_members_delete',
        'commission_members', quality_field_is_lineedit=False,
        quality_db_table_name='commission_member_enum',
        db_function_name='inventory_set_commission_members')


def crc_commission_member_tab_init(dialog, inventory):
    global crc_commission_members_list
    crc_commission_members_list = member_with_quality_tab_init(
        dialog, inventory,
        'crc_commission_members_list', 'crc_commission_members_firstname',
        'crc_commission_members_lastname', 'crc_commission_members_add',
        'crc_commission_members_quality', 'crc_commission_members_delete',
        'crc_commission_members', quality_field_is_lineedit=False,
        quality_db_table_name='crc_commission_member_enum',
        db_function_name='inventory_set_crc_commission_members')


def witness_tab_init(dialog, inventory):
    global witness_list
    witness_list = member_with_quality_tab_init(
        dialog, inventory,
        'witness_list', 'witness_firstname',
        'witness_lastname', 'witness_add',
        'witness_quality', 'witness_delete',
        'witnesses',
        db_function_name='inventory_set_witnesses')
